@@ -1,32 +1,35 @@
 # -*- coding: utf-8 -*-
-from gi.repository import Gtk, GLib, Gio
-
 from threading import Thread
 from traceback import print_exc
 from os import stat
 from os.path import splitext
 from sys import version_info
 from io import open
+from re import compile as re_compile, U as re_U
+from enum import Enum
+
+from gi.repository import Gtk, GLib, Gio
 
 from formiko.vim import VimEditor
-from formiko.sourceview import SourceView
-from formiko.renderer import Renderer, EXTS
+from formiko.sourceview import SourceView, View as GtkSourceView
+from formiko.renderer import Renderer, EXTS, WebView as GtkWebView
 from formiko.dialogs import QuitDialogWithoutSave, FileOpenDialog, \
     FileSaveDialog
 from formiko.preferences import Preferences
-from formiko.user import UserCache, UserPreferences
+from formiko.user import UserCache, UserPreferences, View
 from formiko.icons import icon_list
 from formiko.status_menu import Statusbar
 from formiko.editor_actions import EditorActionGroup
 from formiko.widgets import IconButton
 
-NOT_SAVED_NAME = 'Not Saved Document'
+NOT_SAVED_NAME = 'Untitled Document'
+RE_WORD = re_compile(r'([\w]+)', re_U)
+RE_CHAR = re_compile(r'[\w \t\.,\?\(\)"\']', re_U)
 
 
-class View:
-    EDITOR = 1
-    PREVIEW = 2
-    BOTH = 3
+class SearchWay(Enum):
+    NEXT = 0
+    PREVIOUS = 1
 
 
 class AppWindow(Gtk.ApplicationWindow):
@@ -34,6 +37,8 @@ class AppWindow(Gtk.ApplicationWindow):
         assert editor in ('vim', 'source', None)
         self.runing = True
         self.editor_type = editor
+        self.focused = None
+        self.search_way = SearchWay.NEXT
         self.cache = UserCache()
         self.preferences = UserPreferences()
         super(AppWindow, self).__init__()
@@ -78,15 +83,34 @@ class AppWindow(Gtk.ApplicationWindow):
         action.connect("activate", self.on_close_window)
         self.add_action(action)
 
+        action = Gio.SimpleAction.new("find-in-document", None)
+        action.connect("activate", self.on_find_in_document)
+        self.add_action(action)
+
+        action = Gio.SimpleAction.new("find-next-match", None)
+        action.connect("activate", self.on_find_next_match)
+        self.add_action(action)
+
+        action = Gio.SimpleAction.new("find-previous-match", None)
+        action.connect("activate", self.on_find_previous_match)
+        self.add_action(action)
+
+        self.refresh_preview_action = Gio.SimpleAction.new(
+            "refresh-preview", None)
+        self.refresh_preview_action.connect(
+            "activate", self.on_refresh_preview)
+        self.add_action(self.refresh_preview_action)
+
         pref = self.preferences
 
-        self.show_editor = True
-        self.show_preview = True
-
         self.create_stateful_action(
-            "switch-view-toggle", 'q', View.BOTH, self.on_switch_view_toggle)
+            "switch-view-toggle", 'q', self.cache.view,
+            self.on_switch_view_toggle)
         self.create_stateful_action(
             "change-preview", 'q', pref.preview, self.on_change_preview)
+        self.create_stateful_action(
+            "auto-scroll-toggle", 'b', pref.auto_scroll,
+            self.on_auto_scroll_toggle)
         self.create_stateful_action(
             "change-writer", 's', pref.writer, self.on_change_writer)
         self.create_stateful_action(
@@ -109,6 +133,17 @@ class AppWindow(Gtk.ApplicationWindow):
             self.save_win_state()
             self.destroy()
 
+    def open_document(self, file_path):
+        if self.editor_type == 'source' and \
+                self.get_title() == NOT_SAVED_NAME:
+            self.editor.read_from_file(file_path)
+        else:
+            for window in self.get_application().get_windows():
+                if file_path == window.file_path:
+                    window.present()
+                    return
+            self.get_application().new_window(self.editor_type, file_path)
+
     def on_open_document(self, actions, *params):
         dialog = FileOpenDialog(self)
         dialog.add_filter_plain()
@@ -118,21 +153,16 @@ class AppWindow(Gtk.ApplicationWindow):
         dialog.add_filter_all()
 
         if dialog.run() == Gtk.ResponseType.ACCEPT:
-            if self.editor_type == 'source' and \
-                    self.get_title() == NOT_SAVED_NAME:
-                self.editor.read_from_file(dialog.get_filename())
-            else:
-                self.get_application().new_window(self.editor_type,
-                                                  dialog.get_filename())
+            self.open_document(dialog.get_filename())
         dialog.destroy()
 
     def on_save_document(self, action, *params):
         if self.editor_type == 'source':
-            self.editor.save(self)
+            self.editor.save()
 
     def on_save_document_as(self, action, *params):
         if self.editor_type == 'source':
-            self.editor.save_as(self)
+            self.editor.save_as()
 
     def on_export_document_as(self, action, *params):
         file_name = self.editor.file_name or None
@@ -182,17 +212,21 @@ class AppWindow(Gtk.ApplicationWindow):
             action.set_state(param)
 
         state = param.get_uint16()
+        self.cache.view = state
         if state == View.BOTH:
             self.editor.show()
             self.renderer.show()
+            self.refresh_preview_action.set_enabled(True)
             self.both_toggle_btn.set_active(True)
         elif state == View.EDITOR:
             self.editor.show()
             self.renderer.hide()
+            self.refresh_preview_action.set_enabled(False)
             self.editor_toggle_btn.set_active(True)
         else:
             self.editor.hide()
             self.renderer.show()
+            self.refresh_preview_action.set_enabled(True)
             self.preview_toggle_btn.set_active(True)
 
     def on_change_preview(self, action, param):
@@ -204,12 +238,26 @@ class AppWindow(Gtk.ApplicationWindow):
         orientation = param.get_uint16()
         if self.paned.get_orientation() != orientation:
             self.paned.set_orientation(orientation)
-            if orientation == Gtk.Orientation.HORIZONTAL:
-                self.paned.set_position(self.paned.get_allocated_width()/2)
-            else:
-                self.paned.set_position(self.paned.get_allocated_height()/2)
             self.preferences.preview = orientation
             self.preferences.save()
+            GLib.idle_add(
+                lambda: GLib.timeout_add(100, self.set_position) and False)
+
+    def set_position(self):
+        """Set position after change orientation.
+
+        This must be do after some timeout. It is HARD FIX of gtk error
+        https://gitlab.gnome.org/GNOME/gtk/issues/1959
+        """
+        if self.paned.get_orientation() == Gtk.Orientation.VERTICAL:
+            self.paned.set_position(self.paned.get_allocated_height()/2)
+        else:
+            self.paned.set_position(self.paned.get_allocated_width()/2)
+
+    def on_auto_scroll_toggle(self, action, param):
+        auto_scroll = not self.preferences.auto_scroll
+        self.preferences.auto_scroll = auto_scroll
+        self.preferences.save()
 
     def on_change_parser(self, action, param):
         if action.get_state() != param:
@@ -223,6 +271,10 @@ class AppWindow(Gtk.ApplicationWindow):
         parser = EXTS.get(ext, self.preferences.parser)
         self.pref_menu.set_parser(parser)
 
+    def on_scroll_changed(self, widget, position):
+        if self.preferences.auto_scroll:
+            self.renderer.scroll_to_position(position)
+
     def on_change_writer(self, action, param):
         if action.get_state() != param:
             action.set_state(param)
@@ -249,6 +301,80 @@ class AppWindow(Gtk.ApplicationWindow):
             self.renderer.set_style('')
         self.preferences.save()
 
+    def on_find_in_document(self, action, *param):
+        if self.editor_type != 'source' and not self.renderer.props.visible:
+            return      # works only with source view or renderer
+
+        if self.search.get_search_mode():
+            if self.search_way == SearchWay.NEXT:
+                self.on_find_next_match(action, *param)
+            else:
+                self.on_find_previous_match(action, *param)
+        else:
+            self.search_way = SearchWay.NEXT
+            self.focused = self.get_focus()
+            self.search.set_search_mode(True)
+            self.search_entry.grab_focus()
+
+    def on_search_mode_changed(self, search_bar, param):
+        if not self.search.get_search_mode():
+            if isinstance(self.focused, GtkSourceView):
+                self.editor.stop_search()
+            elif isinstance(self.focused, GtkWebView):
+                self.renderer.stop_search()
+            elif self.editor_type == "source" and self.editor.props.visible:
+                self.editor.stop_search()
+            elif self.renderer.props.visible:
+                self.renderer.stop_search()
+
+            self.focused.grab_focus()
+            self.focused = None
+
+    def on_search_mode_text(self, search_entry, param):
+        if self.search_way == SearchWay.NEXT:
+            res = self.on_find_next_match(None, param)
+        else:
+            res = self.on_find_previous_match(None, param)
+
+        ctx = self.search_entry.get_style_context()
+        if not res and self.search_entry.get_text():
+            Gtk.StyleContext.add_class(ctx, "error")
+        else:
+            Gtk.StyleContext.remove_class(ctx, "error")
+
+    def on_find_next_match(self, action, *params):
+        res = False
+        if self.search.get_search_mode():
+            text = self.search_entry.get_text()
+
+            if isinstance(self.focused, GtkSourceView):
+                res = self.editor.do_next_match(text)
+            elif isinstance(self.focused, GtkWebView):
+                res = self.renderer.do_next_match(text)
+            elif self.editor_type == "source" and self.editor.props.visible:
+                res = self.editor.do_next_match(text)
+            elif self.renderer.props.visible:
+                res = self.renderer.do_next_match(text)
+        return res
+
+    def on_find_previous_match(self, action, *params):
+        res = False
+        if self.search.get_search_mode():
+            text = self.search_entry.get_text()
+
+            if isinstance(self.focused, GtkSourceView):
+                res = self.editor.do_previous_match(text)
+            elif isinstance(self.focused, GtkWebView):
+                res = self.renderer.do_previous_match(text)
+            elif self.editor_type == "source" and self.editor.props.visible:
+                res = self.editor.do_previous_match(text)
+            elif self.renderer.props.visible:
+                res = self.renderer.do_previous_match(text)
+        return res
+
+    def on_refresh_preview(self, action, *params):
+        self.check_in_thread(True)
+
     def ask_if_modified(self):
         if self.editor_type:
             if self.editor.is_modified:
@@ -299,26 +425,33 @@ class AppWindow(Gtk.ApplicationWindow):
         btn.set_tooltip_text("Preferences")
         headerbar.pack_end(btn)
 
-        if self.editor_type != 'preview':
+        headerbar.pack_end(IconButton(symbol="view-refresh-symbolic",
+                                      tooltip="Refresh preview",
+                                      action_name="win.refresh-preview"))
+
+        if self.editor_type:
             btn_box = Gtk.ButtonBox.new(orientation=Gtk.Orientation.HORIZONTAL)
             Gtk.StyleContext.add_class(btn_box.get_style_context(), "linked")
 
             self.editor_toggle_btn = Gtk.ToggleButton(
-                label="Editor",
+                label="_Editor",
+                use_underline=True,
                 action_name="win.switch-view-toggle",
                 action_target=GLib.Variant('q', View.EDITOR))
             self.editor_toggle_btn.set_tooltip_text("Show Editor")
             btn_box.pack_start(self.editor_toggle_btn, True, True, 0)
 
             self.preview_toggle_btn = Gtk.ToggleButton(
-                label="Preview",
+                label="_Preview",
+                use_underline=True,
                 action_name="win.switch-view-toggle",
                 action_target=GLib.Variant('q', View.PREVIEW))
             self.preview_toggle_btn.set_tooltip_text("Show Web Preview")
             btn_box.pack_start(self.preview_toggle_btn, True, True, 0)
 
             self.both_toggle_btn = Gtk.ToggleButton(
-                label="Both",
+                label="_Both",
+                use_underline=True,
                 action_name="win.switch-view-toggle",
                 action_target=GLib.Variant('q', View.BOTH))
             self.both_toggle_btn.set_tooltip_text(
@@ -339,55 +472,106 @@ class AppWindow(Gtk.ApplicationWindow):
     def fill_panned(self, file_name):
         if self.editor_type == 'vim':
             self.editor = VimEditor(self, file_name)
-            self.editor.connect("file_type", self.on_file_type)
         else:
-            self.editor = SourceView(self.preferences, "editor.spell-lang")
+            self.editor = SourceView(self,
+                                     self.preferences,
+                                     "editor.spell-lang")
             self.insert_action_group("editor",
                                      EditorActionGroup(self.editor,
                                                        self.renderer,
                                                        self.preferences))
-            self.editor.connect("file_type", self.on_file_type)
             if file_name:
                 self.editor.read_from_file(file_name)
+
+        self.editor.connect("file-type", self.on_file_type)
+        self.editor.connect("scroll-changed", self.on_scroll_changed)
+
         self.paned.pack1(self.editor, True, False)
         self.paned.pack2(self.renderer, True, False)
 
+        if self.cache.view == View.EDITOR:
+            self.renderer.show_all()
+            self.renderer.hide()
+            self.renderer.set_no_show_all(True)
+            self.refresh_preview_action.set_enabled(False)
+        elif self.cache.view == View.PREVIEW:
+            self.editor.show_all()
+            self.editor.hide()
+            self.editor.set_no_show_all(True)
+
     def layout(self, file_name):
         self.set_default_size(self.cache.width, self.cache.height)
         box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
         self.add(box)
 
+        overlay = Gtk.Overlay()
+        box.pack_start(overlay, True, True, 0)
+
         if self.editor_type:
             self.paned = Gtk.Paned(orientation=self.preferences.preview,
                                    position=self.cache.paned)
-            box.pack_start(self.paned, True, True, 0)
+            overlay.add_overlay(self.paned)
             self.fill_panned(file_name)
         else:
-            self.file_name = file_name
+            self.__file_name = file_name
             self.set_title(file_name)
-            box.pack_start(self.renderer, True, True, 0)
+            overlay.add_overlay(self.renderer)
 
         if self.cache.is_maximized:
             self.maximize()
 
         if self.editor_type == 'source':
-            box.pack_end(Statusbar(self.preferences.editor), False, True, 0)
+            self.status_bar = Statusbar(self.preferences.editor)
+            box.pack_end(self.status_bar, False, True, 0)
 
-    def check_in_thread(self):
+        self.search = Gtk.SearchBar()
+        self.search.set_show_close_button(False)
+        self.search.set_halign(Gtk.Align.CENTER)
+        self.search.set_valign(Gtk.Align.START)
+        self.search.connect("notify::search-mode-enabled",
+                            self.on_search_mode_changed)
+        overlay.add_overlay(self.search)
+
+        sbox = Gtk.Box(Gtk.Orientation.HORIZONTAL, 0)
+        Gtk.StyleContext.add_class(sbox.get_style_context(), "linked")
+        self.search.add(sbox)
+
+        self.search_entry = Gtk.SearchEntry()
+        self.search_entry.set_width_chars(30)
+        sbox.pack_start(self.search_entry, True, True, 0)
+        self.search.connect_entry(self.search_entry)
+        self.search_entry.connect("notify::text",
+                                  self.on_search_mode_text)
+
+        prev_button = IconButton(
+                symbol="go-previous-symbolic",
+                tooltip="Previeous search",
+                action_name="win.find-previous-match",
+                focus_on_click=False)
+        # set_focus_on_click(False)
+        sbox.pack_start(prev_button, False, False, 0)
+        next_button = IconButton(
+                symbol="go-next-symbolic",
+                tooltip="Next search",
+                action_name="win.find-next-match",
+                focus_on_click=False)
+        sbox.pack_start(next_button, False, False, 0)
+
+    def check_in_thread(self, force=False):
         if self.runing:
             if self.editor_type == 'vim':
-                thread = Thread(target=self.refresh_from_vim)
+                thread = Thread(target=self.refresh_from_vim, args=(force,))
                 thread.start()
             elif self.editor_type == 'source':
-                GLib.idle_add(self.refresh_from_source)
+                GLib.idle_add(self.refresh_from_source, force)
             else:   # self.editor = None
-                GLib.idle_add(self.refresh_from_file)
+                GLib.idle_add(self.refresh_from_file, force)
 
     def not_running(self):
         if not self.runing:
             raise SystemExit(0)
 
-    def refresh_from_vim(self):
+    def refresh_from_vim(self, force):
         another_file = False
         try:
             star = '*' if self.editor.is_modified else ''
@@ -399,7 +583,7 @@ class AppWindow(Gtk.ApplicationWindow):
             self.not_running()
             last_changes = self.editor.get_vim_changes()
 
-            if last_changes > self.__last_changes or another_file:
+            if force or last_changes > self.__last_changes or another_file:
                 self.__last_changes = last_changes
                 self.not_running()
                 lines = self.editor.get_vim_lines()
@@ -421,10 +605,12 @@ class AppWindow(Gtk.ApplicationWindow):
         except BaseException:
             print_exc()
 
-    def refresh_from_source(self):
+    def refresh_from_source(self, force):
         try:
             modified = self.editor.is_modified
-            self.lookup_action("save-document").set_enabled(modified)
+            action = self.lookup_action("save-document")
+            if action:  # sometimes when closing window action is None
+                action.set_enabled(modified)
 
             star = '*' if modified else ''
             title = star + (self.editor.file_name or NOT_SAVED_NAME)
@@ -432,22 +618,39 @@ class AppWindow(Gtk.ApplicationWindow):
                 self.set_title(title)
 
             last_changes = self.editor.changes
-            if last_changes > self.__last_changes:
+            if force or last_changes > self.__last_changes:
                 self.__last_changes = last_changes
-                self.renderer.render(self.editor.text, self.editor.file_path,
+                text = self.editor.text
+
+                words_count = 0
+                for w in RE_WORD.finditer(text):
+                    words_count += 1
+                self.status_bar.set_words_count(words_count)
+
+                chars_count = 0
+                for c in RE_CHAR.finditer(text):
+                    chars_count += 1
+                self.status_bar.set_chars_count(chars_count)
+
+                self.renderer.render(text, self.editor.file_path,
                                      self.editor.position)
             GLib.timeout_add(100, self.check_in_thread)
         except BaseException:
             print_exc()
 
-    def refresh_from_file(self):
+    def refresh_from_file(self, force):
         try:
-            last_changes = stat(self.file_name).st_ctime
-            if last_changes > self.__last_changes:
+            last_changes = stat(self.__file_name).st_ctime
+            if force or last_changes > self.__last_changes:
                 self.__last_changes = last_changes
-                with open(self.file_name) as source:
+                with open(self.__file_name) as source:
                     buff = source.read()
-                    self.renderer.render(buff, self.file_name)
+                    self.renderer.render(buff, self.__file_name,
+                                         self.renderer.position)
         except BaseException:
             print_exc()
         GLib.timeout_add(500, self.check_in_thread)
+
+    @property
+    def file_path(self):
+        return self.editor.file_path if self.editor_type else self.__file_name
